@@ -1,5 +1,5 @@
 #!/bin/sh -e
-# Written by Luke Dashjr in 2012
+# Written by Luke Dashjr in 2012-2014
 # This program is released under the terms of the Creative Commons "CC0 1.0 Universal" license and/or copyright waiver.
 
 bs_dir="$(dirname "$0")"
@@ -23,3 +23,8 @@ echo 'Running autoreconf -if...'
 	rm -f aclocal.m4 ltmain.sh
 	autoreconf -if ${AC_FLAGS}
 )
+
+echo 'Updating version.h...'
+if ./gen-version.sh >version.h.new; then
+	cmp version.h version.h.new && rm version.h.new || mv version.h.new version.h
+fi
